import unittest
from pathlib import Path
from denomme.name import person_name_component
from spacy.lang.xx import MultiLanguage

class TestPersonName(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.nlp = MultiLanguage()
        self.nlp.add_pipe("denomme")

    def test_single_name(self):
        doc = self.nlp("Hi my name is Meghana S.R Bhange")
        self.assertListEqual([name.text for name in doc._.person_name], ["Meghana S.R Bhange"])
    
    def test_single_word_names(self):
        doc = self.nlp("Meghana")
        self.assertListEqual([name.text for name in doc._.person_name], ["Meghana"])


    def test_two_names(self):
        doc = self.nlp("Hi my name is Meghana S.R Bhange and I want to talk to Ketaki Ambadkar")
        self.assertListEqual([name.text for name in doc._.person_name], ["Meghana S.R Bhange", "Ketaki Ambadkar"])
    
    def test_no_names(self):
        doc = self.nlp("Hi, I want to talk to someone?")
        self.assertListEqual([name.text for name in doc._.person_name], [])

    def test_city_names(self):
        doc = self.nlp("Hi, I want to book a flight to London")
        self.assertListEqual([name.text for name in doc._.person_name], [])
        
    @classmethod
    def tearDownClass(self):
        pass


import logging

LOGGER = logging.Logger("denomme")
file_handler = logging.FileHandler("denomme.log")
stream_handler = logging.StreamHandler()

stream_formatter = logging.Formatter("%(asctime)-15s %(levelname)-8s %(message)s")
file_formatter = logging.Formatter(
    "{'time':'%(asctime)s', 'name': '%(name)s', \
    'level': '%(levelname)s', 'message': '%(message)s'}"
)

file_handler.setFormatter(file_formatter)
stream_handler.setFormatter(stream_formatter)

LOGGER.addHandler(file_handler)
LOGGER.addHandler(stream_handler)



import spacy
from spacy.language import Language
from spacy.tokens import Doc, Span


@Language.factory("denomme")
def person_name_component(nlp: Language, name: str)->Doc:
    """denomme component which imports the name-detection
    module and also combines the compund name
    Args:
        nlp (Language): spaCy Language Module
        name (str): name of the component
    Returns:
        Doc : spacy doc
    """
    return DenommeComponent(nlp)


class DenommeComponent:
    def __init__(self, nlp: Language):
        """Name detection component which extends the 
        name-ner-detection and combines compound names
        into one name that can be found under the extension
        person_name
        eg.
        doc._.person_name will return Span for the detected 
        person names in the doc
        Args:
            nlp (Language): spaCy Language Module
        """
        self.name_ner = spacy.load("xx_denomme")
        if not Doc.has_extension("person_name"):
            Doc.set_extension("person_name", default=[])

    def __call__(self, doc: Doc) -> Doc:
        name_doc = self.name_ner(f"My name is {doc.text}")
        # TODO Hack, need to fix in next training iteration 
        names = []
        with name_doc.retokenize() as retokenizer:
          for ent in name_doc.ents:
              retokenizer.merge(ent)
        for ent in name_doc.ents:
            end_search = False
            count = 0
            if ent.label_ == "S-PER":
                start = ent.start-3
                end = ent.end-3
                span = doc[start:end]
                name = doc.char_span(span.start_char, span.end_char, label="PERSON")
                names.append(name)
            elif ent.label_ == "B-PER":
                try:
                    next_token = name_doc[ent.start+1]
                    if next_token.ent_type_ == "B-PER":
                        end_search = True
                except:
                    end_search = True
                start = ent.start-3
                prev_token = doc[start-1]
                if prev_token.text.title() in ("Dr", "Dr.", "Mr", "Mr.", "Ms", "Ms.", "Mrs", "Mrs."):
                    start = start - 1
                while not (end_search):
                    count += 1
                    try:
                        next_token = name_doc[ent.start + count]
                    except Exception as e:
                        end_search = True
                    if count == 3 or next_token.ent_type_ == "E-PER":
                        end_search = True
                    end = next_token.i-2
                span = doc[start:end]
                name = doc.char_span(span.start_char, span.end_char, label="PERSON")
                names.append(name)
        doc._.person_name = names
        return doc


"""Convert entity annotation from spaCy v2 TRAIN_DATA format to spaCy v3
.spacy format."""
import warnings
from pathlib import Path

import spacy
import srsly
import typer
from spacy.tokens import DocBin

"""Convert entity annotation from spaCy v2 TRAIN_DATA format to spaCy v3
.spacy format."""
import warnings
from pathlib import Path

import spacy
import srsly
import typer
from spacy.tokens import DocBin


def convert(lang: str, input_path: Path, output_path: Path):
    nlp = spacy.blank(lang)
    db = DocBin()
    count = 0
    total_count = 0
    for text, annot in srsly.read_json(input_path):
        total_count += 1
        doc = nlp.make_doc(text)
        ents = []
        for word_dict in annot["entities"]:
            for key in word_dict:
                word = key
                label = word_dict[key]
            start = doc.text.find(word)
            end = start + len(word)
            span = doc.char_span(start, end, label=label)
            if span is None:
                msg = f"Skipping entity [{start}, {end}, {label}] in the following text because the character span '{doc.text[start:end]}' does not align with token boundaries:\n\n{repr(text)}\n"
                count += 1
            else:
                ents.append(span)
        try:
            doc.ents = ents
        except:
            count += 1
        db.add(doc)
    db.to_disk(output_path)
    print(f"Total Skipped : {count} out of {total_count}")


if __name__ == "__main__":
    typer.run(convert)



import json
import logging
import random
import re
import string
from collections import defaultdict
from pathlib import Path
from random import sample
from spacy.lang.xx import MultiLanguage
import fire
import pandas
import spacy
from spacy.tokens import DocBin, Span
from src import LOGGER
from tqdm import tqdm

LOGGER.info("Generate Sythetic Names")

nlp = MultiLanguage()


def convert(out_dir: Path, files: list):
    for f in files:
        output_filename = out_dir / f"{f.name[:-4]}.json"
        LOGGER.info(f"Saving in {output_filename}")
        all_sentences = f.open("r").read().split("\n\n")
        entity_tags = ("B-PER", "S-PER", "I-PER", "E-PER")
        spacy_sentence_data = []
        for sentence in tqdm(all_sentences):
            try:
                subwords = str(sentence).split("\n")
                entity_tag_relation = []
                sent = []
                for phrase in subwords:
                    texts = phrase.split(" ")
                    word = texts[0]
                    entity = texts[1]
                    sent.append(word)
                    if entity in entity_tags:
                        entity_tag_relation.append({word: entity})
                output_sentence = " ".join(sent)
                output_dict = {"entities": entity_tag_relation}
                spacy_sentence = [output_sentence, output_dict]
                spacy_sentence_data.append(spacy_sentence)
            except:
                pass
        json.dump(spacy_sentence_data, output_filename.open("w"), indent=2)


def convert_to_spacy_format(
    modified_ner_dir: str = "modified_wiki_ner",
    out_dir: str = "spacy_ner_data",
    data_path: str = "./assets",
):
    LOGGER.info(f"==========RECIEVED INPUT============")
    LOGGER.info(f"modified_ner_dir\t\t{modified_ner_dir}")
    LOGGER.info(f"out_dir Path\t\t{out_dir}")
    LOGGER.info(f"data_path\t\t{data_path}")
    LOGGER.info(f"====================================")
    data_path = Path(data_path)
    out_dir = data_path / out_dir
    modified_ner_dir_path = data_path / modified_ner_dir
    p = modified_ner_dir_path.glob("**/*.txt")
    files = [x for x in p if x.is_file()]
    LOGGER.info("======Found following files=====")
    for file in files:
        LOGGER.info(file)
    LOGGER.info("===============================")
    LOGGER.info("Converting to SpaCy Format...")
    convert(out_dir=out_dir, files=files)
    LOGGER.info("Done.")


if __name__ == "__main__":
    fire.Fire(convert_to_spacy_format)



#!/usr/bin/env python3

import argparse
import io
import json
import sys

if sys.stdin:
    input_stream = io.TextIOWrapper(sys.stdin.buffer, encoding="utf-8")
output_stream = io.TextIOWrapper(sys.stdout.buffer, encoding="utf-8")

parser = argparse.ArgumentParser()
parser.add_argument(
    "-t", "--textconv", action="store_true", help="Print results to output"
)
parser.add_argument(
    "-d", "--doc-mode", action="store_true", help="fastai docs nb-specific strip out"
)
parser.add_argument("files", nargs="*", help="Files to strip output from")
args = parser.parse_args()

# define which fields need to be kept:
cell_metadata_keep_code = []
cell_metadata_keep_docs = ["hide_input"]
nb_metadata_keep = ["kernelspec", "jekyll"]


def clean_cell_outputs(o):
    if "execution_count" in o:
        o["execution_count"] = None


### filter for doc nb cells ###
# 1. reset execution_count (in cell and cell's outputs field)
# 2. keep only cell_metadata_keep_doc fields


def clean_cell_docs(o):
    if "execution_count" in o:
        o["execution_count"] = None
    if "outputs" in o:
        for l in o["outputs"]:
            clean_cell_outputs(l)

    o["metadata"] = {
        k: o["metadata"][k]
        for k in o["metadata"].keys()
        if k in cell_metadata_keep_docs
    }
    return o


### filter for code nb cells ###
# 1. reset execution_count
# 2. delete cell's metadata
# 3. delete cell's outputs


def clean_cell_code(o):
    if "execution_count" in o:
        o["execution_count"] = None
    if "outputs" in o:
        o["outputs"] = []
    o["metadata"] = {}
    return o


# optimize runtime
clean_cell = clean_cell_code if not args.doc_mode else clean_cell_docs

### filter for nb top level entries ###
# 1. keep only nb_metadata_keep fields
# 2. the other rules apply based on clean_cell alias


def clean_nb(s):
    s["cells"] = [clean_cell(o) for o in s["cells"]]
    s["metadata"] = {
        k: s["metadata"][k] for k in s["metadata"].keys() if k in nb_metadata_keep
    }


for filename in args.files:
    if not filename.endswith(".ipynb"):
        continue
    with io.open(filename, "r", encoding="utf-8") as f:
        s = json.load(f)
    clean_nb(s)
    x = json.dumps(s, sort_keys=True, indent=1, ensure_ascii=False)

    if args.textconv:
        # XXX: if there is more than one file, this is probably wrong
        output_stream.write(x)
        output_stream.write("\n")
        output_stream.flush()
    else:
        with io.open(filename, "w", encoding="utf-8") as f:
            f.write(x)
            f.write("\n")

# implied textconv mode
if not args.files and input_stream:
    s = json.load(input_stream)
    clean_nb(s)
    x = json.dumps(s, sort_keys=True, indent=1, ensure_ascii=False)
    output_stream.write(x)
    output_stream.write("\n")
    output_stream.flush()



from setuptools import setup
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='denomme',
    version='0.3.9',    
    description="Name detection using spacy",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://github.com/meghanabhange/denomme.git",
    author='Meghana Bhange',
    author_email='meghanabhange@hey.com',
    license='MIT',
    packages=['denomme'],
    install_requires=['spacy==3.0.5',
                      'spacy-transformers==1.0.1',  
                      'sentencepiece==0.1.95'
                      ],
     dependency_links=  [
                        'https://denomme.s3.us-east-2.amazonaws.com/xx_denomme-0.3.1/dist/xx_denomme-0.3.1.tar.gz'
                        ]
)
